@@ -32,6 +32,16 @@ The Airbyte CDK provides a set of classes that help you work with the Airbyte pr
 - `airbyte_cdk.models.airbyte_protocol`
 - `airbyte_cdk.models.airbyte_protocol_serializers`
 
+## Using the CLI (`airbyte_cdk.cli`)
+
+The Airbyte CDK provides two command-line interfaces (CLIs) for interacting with the framework.
+
+- `airbyte-cdk`: This is the main CLI for the Airbyte CDK. It provides commands for building
+  and testing connectors, as well as other utilities. See the `airbyte_cdk.cli.airbyte_cdk` module
+  for more details.
+- `source-declarative-manifest`: This command allows you to run declarative manifests directly.
+  See the `airbyte_cdk.cli.source_declarative_manifest` module for more details.
+
 ---
 
 API Reference
