@@ -1,17 +1,21 @@
 #
-# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
+# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
 #
 
 import builtins
 import datetime
+import re
 import typing
 from typing import Optional, Union
+from urllib.parse import quote_plus
 
 import isodate
 import pytz
 from dateutil import parser
 from isodate import parse_duration
 
+from airbyte_cdk.sources.declarative.datetime.datetime_parser import DatetimeParser
+
 """
 This file contains macros that can be evaluated by a `JinjaInterpolation` object
 """
@@ -63,10 +67,24 @@ def timestamp(dt: Union[float, str]) -> Union[int, float]:
     if isinstance(dt, (int, float)):
         return int(dt)
     else:
-        return _str_to_datetime(dt).astimezone(pytz.utc).timestamp()
+        return str_to_datetime(dt).astimezone(pytz.utc).timestamp()
 
 
-def _str_to_datetime(s: str) -> datetime.datetime:
+def str_to_datetime(s: str) -> datetime.datetime:
+    """
+    Converts a string to a datetime object with UTC timezone
+
+    If the input string does not contain timezone information, UTC is assumed.
+    Supports both basic date strings like "2022-01-14" and datetime strings with optional timezone
+    like "2022-01-01T13:45:30+00:00".
+
+    Usage:
+    `"{{ str_to_datetime('2022-01-14') }}"`
+
+    :param s: string to parse as datetime
+    :return: datetime object in UTC timezone
+    """
+
     parsed_date = parser.isoparse(s)
     if not parsed_date.tzinfo:
         # Assume UTC if the input does not contain a timezone
@@ -140,7 +158,7 @@ def duration(datestring: str) -> Union[datetime.timedelta, isodate.Duration]:
 
 
 def format_datetime(
-    dt: Union[str, datetime.datetime], format: str, input_format: Optional[str] = None
+    dt: Union[str, datetime.datetime, int], format: str, input_format: Optional[str] = None
 ) -> str:
     """
     Converts datetime to another format
@@ -154,12 +172,39 @@ def format_datetime(
     """
     if isinstance(dt, datetime.datetime):
         return dt.strftime(format)
-    dt_datetime = (
-        datetime.datetime.strptime(dt, input_format) if input_format else _str_to_datetime(dt)
-    )
-    if format == "%s":
-        return str(int(dt_datetime.timestamp()))
-    return dt_datetime.strftime(format)
+
+    if isinstance(dt, int):
+        dt_datetime = DatetimeParser().parse(dt, input_format if input_format else "%s")
+    else:
+        dt_datetime = (
+            datetime.datetime.strptime(dt, input_format) if input_format else str_to_datetime(dt)
+        )
+    if dt_datetime.tzinfo is None:
+        dt_datetime = dt_datetime.replace(tzinfo=pytz.utc)
+    return DatetimeParser().format(dt=dt_datetime, format=format)
+
+
+def sanitize_url(value: str) -> str:
+    """
+    Sanitizes a value by via urllib.parse.quote_plus
+
+    Usage:
+    `"{{ sanitize_url('https://example.com/path?query=value') }}"`
+    """
+    sanitization_strategy = quote_plus
+    return sanitization_strategy(value)
+
+
+def camel_case_to_snake_case(value: str) -> str:
+    """
+     Converts CamelCase strings to snake_case format
+
+     Usage:
+    `"{{ camel_case_to_snake_case('CamelCase') }}"`
+     :param value: string to convert from CamelCase to snake_case
+     :return: snake_case formatted string
+    """
+    return re.sub(r"(?<!^)(?=[A-Z])", "_", value).lower()
 
 
 _macros_list = [
@@ -172,5 +217,8 @@ _macros_list = [
     duration,
     format_datetime,
     today_with_timezone,
+    str_to_datetime,
+    sanitize_url,
+    camel_case_to_snake_case,
 ]
 macros = {f.__name__: f for f in _macros_list}
